<?php

namespace App\Commands;

use App\Commands\Concerns\ClonesProjects;
use App\Installers\CopyProjectTask;
use App\Installers\CreateDatabaseTask;
use App\Installers\InitialiseProject;
use App\Projects\ProjectBase;
use LaravelZero\Framework\Commands\Command;

class InstallProject extends Command
{
    use ClonesProjects;

    /**
     * The signature of the command.
     *
     * @var string
     */
    protected $signature = 'install {name : Name of the application you want to install (required)}';

    /**
     * The description of the command.
     *
     * @var string
     */
    protected $description = 'Install an existing project.';

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $projectName = $this->argument('name');
        $url = 'git@git.drp.digital:' . $projectName . '.git';
        $project = new ProjectBase($projectName, $projectName, $url);

        $this->cloneProject($project);
        $this->task('Installing ' . $projectName, new CopyProjectTask($project, $projectName));
        $this->task('Create database', new CreateDatabaseTask($projectName));
        $this->title('Running setup commands');
        (new InitialiseProject($projectName, $this))();

        $this->output->success('Successfully set up your project!');
    }
}
