<?php

namespace App\Commands;

use App\Commands\Concerns\ClonesProjects;
use App\Git\ResetGitTask;
use App\Installers\CopyProjectTask;
use App\Projects\ProjectBases;
use LaravelZero\Framework\Commands\Command;

class NewProject extends Command
{
    use ClonesProjects;

    /**
     * The signature of the command.
     *
     * @var string
     */
    protected $signature = 'new {name : Name of the application you want to create (required)} {--type= : The type of application to install (optional)}';

    /**
     * The description of the command.
     *
     * @var string
     */
    protected $description = 'Create a new an application';

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $type = $this->decideProjectType();
        $projectBase = ProjectBases::findByType($type);
        $projectName = $this->argument('name');

        $this->cloneProject($projectBase);
        $this->task('Creating your new project', new CopyProjectTask($projectBase, $projectName));
        $this->task('Resetting the git history', new ResetGitTask($projectName));

        // Run setup commands
        // - Look for json file?
        // - Perform actions required
    }

    private function decideProjectType(): string
    {
        $type = (string) $this->option('type');

        if (blank($type)) {
            $type = $this->menu(
                'What type of project do you want to create?',
                ProjectBases::all()->pluck('label', 'type')->toArray()
            )->disableDefaultItems()->open();
        }

        return $type;
    }
}
