<?php

namespace App\Installers;

use App\Git\Cloneable;
use Illuminate\Support\Facades\Storage;
use Symfony\Component\Finder\Finder;
use Symfony\Component\Finder\SplFileInfo;

class CopyProjectTask
{
    /**
     * @var \App\Git\Cloneable
     */
    private $cloneable;

    /**
     * @var string
     */
    private $destination;

    public function __construct(Cloneable $cloneable, string $destination)
    {
        $this->cloneable = $cloneable;
        $this->destination = $destination;
    }

    public function __invoke()
    {
        $allFiles = Storage::disk('git-clone')->allFiles($this->cloneable->getFolder());
        foreach ($allFiles as $filePath) {
            $stream = Storage::disk('git-clone')->readStream($filePath);

            Storage::disk('local')->put($this->destination . '/' . $this->stripFolder($filePath), $stream);
        }

        $this->applySymlinks();
    }

    private function applySymlinks(): void
    {
        $finder = (new Finder)->in(Storage::disk('git-clone')->path($this->cloneable->getFolder()))->files();
        collect($finder)->filter(function (SplFileInfo $file) {
            return $file->isLink();
        })->each(function (SplFileInfo $file) {
            $linkTarget = $file->getLinkTarget();
            $directory = Storage::disk('local')->path($this->destination . '/' . $file->getRelativePath());

            $original = getcwd();
            chdir($directory);
            symlink($linkTarget, $file->getFilename());
            chdir($original);
        });
    }

    private function stripFolder(string $filePath): string
    {
        return str_replace($this->cloneable->getFolder() . '/', '', $filePath);
    }
}
